# -*- coding: utf-8 -*-

from django.conf import settings
from django.utils.crypto import get_random_string

from django_sendfile import sendfile
import tempfile
import zipfile
import tarfile
import shutil
import os
import stat

__all__ = ["TarArchive", "ZipArchive"]


class MKArchive(object):
    def __init__(self):
        self.tmp_directory = tempfile.mkdtemp(prefix="billetterie-export-")
        self.archive_dir = os.path.join(settings.MEDIA_ROOT, "protected", "export_archive")
        if not os.path.exists(self.archive_dir):
            os.mkdir(self.archive_dir, stat.S_IRWXU | stat.S_IRWXG)
            os.chown(self.archive_dir, os.geteuid(), os.getegid())

    def _enter_check(self):
        assert self.tmp_directory is not None, "Archive is closed"

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        self.clean()

    def clean(self):
        if self.tmp_directory is not None:
            shutil.rmtree(self.tmp_directory)
        self.tmp_directory = None

    def addContent(self, content, arcname, bytemode=False):
        self._enter_check()
        fd, path = tempfile.mkstemp(dir=self.tmp_directory, text=content)
        f = os.fdopen(fd, "wb" if bytemode else "w")
        f.write(content)
        f.close()
        self.addFile(path, arcname)
        os.remove(path)


class TarArchive(MKArchive):
    def __init__(self):
        super(TarArchive, self).__init__()
        self.archive_name = os.path.join(self.archive_dir, "{}.tar.gz".format(get_random_string(length=12)))
        self.archive = tarfile.open(self.archive_name, mode="w:gz")
        os.chmod(self.archive_name, stat.S_IRWXU | stat.S_IRWXG)
        os.chown(self.archive_name, os.geteuid(), os.getegid())

    def clean(self):
        if self.archive is not None:
            self.archive.close()
        self.archive = None
        super(TarArchive, self).clean()

    def get_response(self, request, filename):
        self._enter_check()
        self.archive.close()
        self.archive = None
        super(TarArchive, self).clean()
        response = sendfile(
            request, self.archive_name, attachment=True, attachment_filename="{}.tar.gz".format(filename)
        )
        if "X-Sendfile" in response:
            response["X-SENDFILE-TEMPORARY"] = response["X-SENDFILE"]
            del response["X-SENDFILE"]
        return response

    def addFile(self, path, arcname):
        self._enter_check()
        self.archive.add(path, arcname)


class ZipArchive(MKArchive):
    def __init__(self):
        super(ZipArchive, self).__init__()
        self.archive_name = os.path.join(self.archive_dir, "{}.zip".format(get_random_string(length=12)))
        self.archive = zipfile.ZipFile(self.archive_name, mode="w")
        os.chmod(self.archive_name, stat.S_IRWXU | stat.S_IRWXG)
        os.chown(self.archive_name, os.geteuid(), os.getegid())

    def clean(self):
        if self.archive is not None:
            self.archive.close()
        self.archive = None
        super(ZipArchive, self).clean()

    def get_response(self, request, filename):
        self._enter_check()
        self.archive.close()
        self.archive = None
        super(ZipArchive, self).clean()
        response = sendfile(request, self.archive_name, attachment=True, attachment_filename="{}.zip".format(filename))
        if "X-Sendfile" in response:
            response["X-SENDFILE-TEMPORARY"] = response["X-SENDFILE"]
            del response["X-SENDFILE"]
        return response

    def addFile(self, path, arcname):
        self._enter_check()
        self.archive.write(path, arcname)
